
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include "dockapp.h"
#include "xpm/xpm.h"

#define PACKAGE		"wmhs"
#define VERSION		"0.0.1"
#define SIZE	    58
#define WINDOWED_BG "  \tc #AEAAAE"
#define MAX_HISTORY 16
#define CPUNUM_NONE -1


typedef enum { FAN, TEMP } type_mesure;
typedef enum { LIGHTON, LIGHTOFF } light;

Pixmap pixmap;
Pixmap backdrop_on;
Pixmap backdrop_off;
Pixmap backdrop_alarm;
Pixmap backdrop_err;
Pixmap parts;
Pixmap mask;
static char	*display_name = "";
static char	*light_color = NULL;	/* back-light color */
static unsigned update_interval = 1;
static light    backlight = LIGHTOFF;
static type_mesure tmode = TEMP;

static Bool alarm = False;

int port; /* port number to mbmon */

extern Bool get_mbmon (int port, double *ctemp, int *vsys, int *vcpu); 

static unsigned int sys_fan=0;
static unsigned int cpu_fan=0;
static double cpu_temp;

static  unsigned int sf = 0;
static  unsigned int cf = 0;
static  unsigned int ct = 0;

static char **backlight_on_xpm;
static char **backlight_off_xpm;
static char **backlight_alarm_xpm;

/* prototypes */
static void update(void);
static void switch_light(void);

static void draw_fan(int per, int pos);
static void draw_temp(double dper, int pos);

static void parse_arguments(int argc, char **argv);
static void print_help(char *prog);

int main(int argc, char **argv)
{
   XEvent event;
   XpmColorSymbol colors[2] = { {"Back0", NULL, 0}, {"Back1", NULL, 0} };
   int ncolor = 0;

   /* Parse CommandLine */
   parse_arguments(argc, argv);

   if (tmode == FAN) {
     backlight_on_xpm = fan_on_xpm;
     backlight_off_xpm = fan_off_xpm;
     backlight_alarm_xpm = fan_alarm_xpm;
   } else {
     backlight_on_xpm = temp_on_xpm;
     backlight_off_xpm = temp_off_xpm;
     backlight_alarm_xpm = temp_alarm_xpm;
   }


   /* Initialize Application */
   /*   temp_init(configfile); */
   dockapp_open_window(display_name, PACKAGE, SIZE, SIZE, argc, argv);
   dockapp_set_eventmask(ButtonPressMask);

   if(light_color)
   {
      colors[0].pixel = dockapp_getcolor(light_color);
      colors[1].pixel = dockapp_blendedcolor(light_color, -24, -24, -24, 1.0);
      ncolor = 2;
   }

   /* change raw xpm data to pixmap */
   if(dockapp_iswindowed)
      backlight_on_xpm[1] = backlight_off_xpm[1] = backlight_alarm_xpm[1] = WINDOWED_BG;

   if(!dockapp_xpm2pixmap(backlight_on_xpm, &backdrop_on, &mask, colors, ncolor))
   {
      fprintf(stderr, "Error initializing backlit background image.\n");
      exit(1);
   }
   if(!dockapp_xpm2pixmap(backlight_off_xpm, &backdrop_off, NULL, NULL, 0))
   {
      fprintf(stderr, "Error initializing background image.\n");
      exit(1);
   }
   if(!dockapp_xpm2pixmap(backlight_alarm_xpm, &backdrop_alarm, NULL, NULL, 0))
   {
      fprintf(stderr, "Error initializing background image.\n");
      exit(1);
   }
   if(!dockapp_xpm2pixmap(backlight_err_xpm, &backdrop_err, NULL, NULL, 0))
   {
      fprintf(stderr, "Error initializing err background image.\n");
      exit(1);
   }

   if(!dockapp_xpm2pixmap(parts_xpm, &parts, NULL, colors, ncolor))
   {
      fprintf(stderr, "Error initializing parts image.\n");
      exit(1);
   }

   /* shape window */
   if(!dockapp_iswindowed)
      dockapp_setshape(mask, 0, 0);
   if(mask) XFreePixmap(display, mask);

   /* pixmap : draw area */
   pixmap = dockapp_XCreatePixmap(SIZE, SIZE);

   if(backlight == LIGHTON) 
      dockapp_copyarea(backdrop_on, pixmap, 0, 0, SIZE, SIZE, 0, 0);
   else
      dockapp_copyarea(backdrop_off, pixmap, 0, 0, SIZE, SIZE, 0, 0);

   dockapp_set_background(pixmap);
   dockapp_show();

   update();

   /* Main loop */
   while(1)
   {
      if (dockapp_nextevent_or_timeout(&event, update_interval * 1000))
      {
         /* Next Event */
         switch(event.type)
         {
            case ButtonPress:
               switch_light();
               break;
            default: /* make gcc happy */
               break;
         }
      }
      else
      {
         /* Time Out */
         update();
      }
   }

   return 0;
}

/* called by timer */
static void update(void)
{


  if (get_mbmon(port, &cpu_temp, &sys_fan, &cpu_fan)) {
   
    alarm = False;
    if ( tmode == FAN ) {
      if ((cpu_fan == 0  && cf > 0) || (sys_fan == 0 &&  cf > 0)) alarm=True;
      if ( cpu_fan > cf && cf > 0) alarm=True;
      if ( sys_fan > sf && sf > 0) alarm=True;
    } else {
      if ( (int)cpu_temp == 0 && ct > 0) alarm=True;
      if ( cpu_temp > ct && ct > 0) alarm=True;
    }

 
    if (alarm) dockapp_copyarea(backdrop_alarm, pixmap, 0, 0, 58, 58, 0, 0);
    else if (backlight == LIGHTON) 
      dockapp_copyarea(backdrop_on, pixmap, 0, 0, 58, 58, 0, 0);
    else 
      dockapp_copyarea(backdrop_off, pixmap, 0, 0, 58, 58, 0, 0);

    /* draw digit */
    if ( tmode == FAN ) {
      draw_fan(cpu_fan,7);
      draw_fan(sys_fan,34);
    } else {
      draw_temp(cpu_temp,7);
    }
  } else 
    dockapp_copyarea(backdrop_err, pixmap, 0, 0, 58, 58, 0, 0);

  /* show */
  dockapp_copy2window(pixmap);
}

/* called when mouse button pressed */
static void switch_light(void)
{


  /* redraw digit */
  if (get_mbmon(port,&cpu_temp, &sys_fan, &cpu_fan)) {

    alarm = False;

    if ( tmode == FAN ) {
      if ((cpu_fan == 0  && cf > 0) || (sys_fan == 0 &&  cf > 0)) alarm=True;
      if ( cpu_fan > cf && cf > 0) alarm=True;
      if ( sys_fan > sf && sf > 0) alarm=True;
    } else {
      if ( (int)cpu_temp == 0 && ct > 0) alarm=True;
      if ( (int)cpu_temp > ct && ct > 0) alarm=True;
    }
    if (alarm)
      dockapp_copyarea(backdrop_alarm, pixmap, 0, 0, 58, 58, 0, 0);
    else {
      switch (backlight)
	{
	case LIGHTOFF:
	  backlight = LIGHTON;
	  dockapp_copyarea(backdrop_on, pixmap, 0, 0, 58, 58, 0, 0);
	  break;
	case LIGHTON:
	  backlight = LIGHTOFF;
	  dockapp_copyarea(backdrop_off, pixmap, 0, 0, 58, 58, 0, 0);
	  break;
	}
    }

    if ( tmode == FAN ) {
      draw_fan(cpu_fan,7);
      draw_fan(sys_fan,34);
    } else {
      draw_temp(cpu_temp,7);
    }
  } else {

    dockapp_copyarea(backdrop_err, pixmap, 0, 0, 58, 58, 0, 0);
  }
  /* show */
  dockapp_copy2window(pixmap);
}



static void draw_fan(int per, int pos)
{
  int v1000, v100, v10, v1;
   int y = 0;

   if (per < 0) per = 0;

   v1000= per / 1000;
   v100 = ( per  - (v1000 * 1000)) / 100;
   v10  = (per - (v1000*1000) - (v100 * 100)) / 10;
   v1   = (per - (v1000*1000)  - v100 * 100 - v10 * 10);

   if (alarm) y = 49;
   else if ((backlight == LIGHTON)) y = 20;


   /* draw digit */
   dockapp_copyarea(parts, pixmap, v1 * 10, y, 10, 20, 41, pos);
   dockapp_copyarea(parts, pixmap, v10 * 10, y, 10, 20, 29, pos);
   dockapp_copyarea(parts, pixmap, v100 * 10, y, 10, 20, 17, pos);
   dockapp_copyarea(parts, pixmap, v1000 * 10, y, 10, 20,  5, pos);
}


static void draw_temp(double dper, int pos)
{
  int   v100, v10, v1;
   int y = 0;
   int per;

   per = (int) dper;
   if (per < 0) per = 0;
   
   v100 = per / 100;
   v10  = ( per  - (v100 * 100)) / 10;
   v1   = (per  - v100 * 100 - v10 * 10);


    if (alarm) y = 49;
   else if (backlight == LIGHTON) y = 20;

   /* draw digit */

   dockapp_copyarea(parts, pixmap, v1 * 10, y, 10, 20, 29, pos);
   if (v10 > 0) dockapp_copyarea(parts, pixmap, v10 * 10, y, 10, 20, 17, pos);
   if (v100 > 0) dockapp_copyarea(parts, pixmap, v100 * 10, y, 10, 20, 5, pos);
}


static void parse_arguments(int argc, char **argv)
{
   int i;
   int integer;
   for (i = 1; i < argc; i++) 
   {
      if (!strcmp(argv[i], "-v"))
         printf("%s version %s\n", PACKAGE, VERSION), exit(0);
      else if (!strcmp(argv[i], "-d")) 
      {
         display_name = argv[i + 1];
         i++;
      }

      else if (!strcmp(argv[i], "-bl"))
         backlight = LIGHTON;
      else if (!strcmp(argv[i], "-lc")) 
      {
         light_color = argv[i + 1];
         i++;
      }

      else  if (!strcmp(argv[i],"-f") ) tmode = FAN;
      else  if (!strcmp(argv[i],"-t") ) tmode = TEMP;

      else if (!strcmp(argv[i], "-i")) 
      {
         if (argc == i + 1)
            fprintf(stderr, "%s: error parsing argument for option %s\n",
                    argv[0], argv[i]), exit(1);
         if (sscanf(argv[i + 1], "%i", &integer) != 1)
             fprintf(stderr, "%s: error parsing argument for option %s\n",
                     argv[0], argv[i]), exit(1);
         if (integer < 1)
             fprintf(stderr, "%s: argument %s must be >=1\n",
                     argv[0], argv[i]), exit(1);
         update_interval = integer;
         i++;
      }
     else if (!strcmp(argv[i], "-p")) 
      {
         if (argc == i + 1)
            fprintf(stderr, "%s: error parsing argument for option %s\n",
                    argv[0], argv[i]), exit(1);
         if (sscanf(argv[i + 1], "%i", &port) != 1)
             fprintf(stderr, "%s: error parsing argument for option %s\n",
                     argv[0], argv[i]), exit(1);
         if (port < 1)
             fprintf(stderr, "%s: argument %s must be >=1\n",
                     argv[0], argv[i]), exit(1);
         i++;
      }

     else if (!strcmp(argv[i], "-cf")) 
      {
         if (argc == i + 1)
            fprintf(stderr, "%s: error parsing argument for option %s\n",
                    argv[0], argv[i]), exit(1);
         if (sscanf(argv[i + 1], "%i", &cf) != 1)
             fprintf(stderr, "%s: error parsing argument for option %s\n",
                     argv[0], argv[i]), exit(1);
         i++;
      }
     else if (!strcmp(argv[i], "-sf")) 
      {
         if (argc == i + 1)
            fprintf(stderr, "%s: error parsing argument for option %s\n",
                    argv[0], argv[i]), exit(1);
         if (sscanf(argv[i + 1], "%i", &sf) != 1)
             fprintf(stderr, "%s: error parsing argument for option %s\n",
                     argv[0], argv[i]), exit(1);
         i++;
      }
     else if (!strcmp(argv[i], "-ct")) 
      {
         if (argc == i + 1)
            fprintf(stderr, "%s: error parsing argument for option %s\n",
                    argv[0], argv[i]), exit(1);
         if (sscanf(argv[i + 1], "%i", &ct) != 1)
             fprintf(stderr, "%s: error parsing argument for option %s\n",
                     argv[0], argv[i]), exit(1);
         i++;
      }

      else if (!strcmp(argv[i], "-w"))
         dockapp_iswindowed = True;
      else if (!strcmp(argv[i], "-bw"))
         dockapp_isbrokenwm = True;
      else
      {
         print_help(argv[0]), exit(1);
      }
   }
}

static void print_help(char *prog)
{
   printf("Usage : %s [OPTIONS]\n", prog);
   printf("wmhs fans  monitor dockapp\n");
   printf("  -d <string>                    display to use\n");
   printf("  -bl                            turn on back-light\n");
   printf("  -lc <string>                   back-light color(rgb:6E/C6/3B is default)\n");
   printf("  -i <number>                    number of secs between updates (1 is default)\n");
   printf("  -h                             show this help text and exit\n");
   printf("  -v                             show program version and exit\n");
   printf("  -w                             run the application in windowed mode\n");
   printf("  -bw                            activate broken window manager fix\n");
   printf("  -p <number>                    port number.\n");
   printf("  -t                             temperature.\n");
   printf("  -f                             fan speed.\n");
   printf("  -cf                            cpu fan speed max.\n");
   printf("  -sf                             sys fan speed max.\n");
   printf("  -ct                             cpu temp speed.\n");



}
